#!/usr/bin/python
# -*- coding: utf-8 -*-

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
# Author: Tomas "Tojaj" Mlcoch (xtojaj gmail dot com)
# Web: http://tojaj.com
#
# Copyright 2010 Tomas Mlcoch

"""Module provides DataModel class wich inherit from
QAbstractTableModel."""

import target
from PyQt4.QtCore import * 
from PyQt4.QtGui import * 


class DataModel(QAbstractTableModel): 
    """Data model for PyPinger application. 
    Class inherit from QAbstractTableModel.
    Data model implements all necessary methods for usage as model 
    in Qt view classes."""

    def __init__(self, parent=None, *args): 
        QAbstractTableModel.__init__(self, parent, *args) 
        self.items = {}
        self.itemkeys = []
        self.headerdata = target.PingTarget.headerdata()
        self.itemscount = 0
        self.counter = -1

    def __getitem__(self, key):
        return self.items.get(key, None)

    def _oneline_change(self, ikey='', order=-1):
        """Method for emit dataChanged signal if one line is updated."""
        if not ikey and order < 0:
            return

        if not order < 0:
            row = order
        else:
            row = self.itemkeys.index(ikey)

        a = QModelIndex()
        b = QModelIndex()
        a.row = row         
        a.column = 1
        b.row = row
        b.column = (self.columnCount() - 1)
        self.emit(SIGNAL('dataChanged(QModelIndex, QModelIndex)'), a, b)

    def refresh(self, key, seq):
        """Tells to item, under 'key' in model, that we captured 
        its ICMP packet with sequence number 'seq'."""
        try:
            self.items[key].recv(seq)
        except (KeyError):
            return
        self._oneline_change(ikey=key)

    def next(self):
        """Returns tuple containing three items.
        First two are ip address and sequence number for next ICMP packet.
        Third item is boolean information about begin of new cycle."""
        if not self.itemscount:
            return None, None, None
        self.counter = (self.counter + 1) % self.itemscount
        key = self.itemkeys[self.counter]
        newcycle = True if (self.counter + 1 == self.itemscount) else False
        self._oneline_change(order=self.counter)
        return (key, self.items[key].next_seq(), newcycle)

    def addItem(self, item):
        """Puts new item (PingTarget object) into model."""
        if not item.ready:
            return False
        if not self.headerdata:
            self.headerdata = item.headerdata()
        if item.ip in self.items:
            return False

        temp = self.itemkeys[:]
        temp.append(item.ip)
        temp.sort()
        row = temp.index(item.ip)
        self.beginInsertRows(QModelIndex(), row, row)
        self.items[item.ip] = item
        self.itemkeys = sorted(self.items.keys())
        self.itemscount = len(self.itemkeys)
        self.endInsertRows()
        return True

    def delItem(self, row):
        """Removes item with id 'row' from model."""
        if not (0 <= row < self.itemscount):
            return
        self.beginRemoveRows(QModelIndex(), row, row)
        del self.items[self.itemkeys[row]]
        self.itemkeys = sorted(self.items.keys())
        self.itemscount = len(self.itemkeys)
        self.endRemoveRows()
        return True

    def get_names(self):
        """Returns list of item names."""
        names = []
        for key in self.itemkeys:
            names.append(self.items[key].name)
        return names

    def rowCount(self, parent=QModelIndex()): 
        """Returns number of rows in model."""
        return self.itemscount 
 
    def columnCount(self, parent=QModelIndex()):
        """Returns number of columns in model."""
        return len(self.headerdata) 
 
    def data(self, index, role): 
        """Returns data on 'index' (QModelIndex) with 'role' role."""
        if not index.isValid(): 
            return QVariant() 
        if role != Qt.DisplayRole and role != Qt.DecorationRole:
            return QVariant()
        if role == Qt.DecorationRole and index.column() != 0:
            return QVariant()

        if index.column() == 0 and role == Qt.DecorationRole:
            status = QPixmap(10, 10)
            if self.items[self.itemkeys[ index.row()]][1] == target.STAT_UP:
                status.fill(QColor(72, 212, 100))
            else:
                status.fill(QColor(227, 59, 77))
            return status

        return QVariant(self.items[self.itemkeys[ index.row()]][index.column()]) 

    def headerData(self, col, orientation, role):
        """Returns header data for column."""
        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
            return QVariant(self.headerdata[col])
        return QVariant()
