#!/usr/bin/python
# -*- coding: utf-8 -*-

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
# Author: Tomas "Tojaj" Mlcoch (xtojaj gmail dot com)
# Web: http://tojaj.com
#
# Copyright 2010 Tomas Mlcoch


import time
import socket
import logging

STAT_UP = 1
STAT_DOWN = 2
DEFAULT_IP = '127.0.0.1'

LOG_LOST = "{0} - One packet lost"
LOG_DOWN = "{0} - Host is DOWN"
LOG_UP = "{0} - Host is UP"
LOG_SUMMARY = "{0} - Outage duration: {1:.1} minutes"

class PingTarget(object):
    """Target of ping requests."""

    LOG = None

    def __init__(self, inip=DEFAULT_IP):
        if not self.LOG:
            self.LOG = logging.getLogger("pypinger")
        self.name = ''
        self.ip = ''
        self.ready = False
        self.status = STAT_DOWN
        self.seq_num = 0
        self.v_seq_num = 0
        self.last_recv_seq = 0
        self.time = 0
        self.send_time = 0
        self.losted = 0
        self.complete_time = 0
        self.min_delay = 100000
        self.avrg_delay = 0
        self.max_delay = 0
        self.set_ip(inip)

        # logging stuff
        self.outage = False
        self.logged = False
        self.outage_start_time = None

    def __len__(self):
        return 9

    def __getitem__(self, key):
        if key == 0:  # Host
            return self.name
        elif key == 1:  # Link status
            return self.status
        elif key == 2:  # Delay
            if self.status == STAT_DOWN:
                return '-'
            return '<1' if self.time < 1 else str(self.time)
        elif key == 3:  # Sent
            return self.seq_num
        elif key == 4:  # Lost
            return self.losted
        elif key == 5:  # Min
            if self.losted >= (self.seq_num - 1):
                return '-'
            return '<1' if self.min_delay < 1 else str(self.min_delay)
        elif key == 6:  # Avrg
            num = self.seq_num - self.losted
            if num == 0 or num == 1:
                return '-'
            avrg = self.complete_time // num
            return '<1' if avrg < 1 else str(avrg)   
        elif key == 7:  # Max
            if self.losted >= (self.seq_num - 1):
                return '-'
            return '<1' if self.max_delay < 1 else str(self.max_delay)
        elif key == 8: # Reliability
            if self.seq_num <= 1:
                return 0
            return int(100 - ((float(self.losted) / (self.seq_num -1)) * 100))

    @staticmethod
    def headerdata():
        """Returns list of headers."""
        header = [
        'Host',
        'Status',
        'Delay [ms]',
        'Sent',
        'Lost',
        'Min. [ms]',
        'Avrg. [ms]',
        'Max. [ms]',
        '% Reliability',
        ]
        return header

    def set_ip(self, inip):
        """Set target IP address."""
        if not inip:
            return
        self.name = inip
        self.ip = self.ipv4_validator(inip)
        if self.ip:
            self.ready = True

    @staticmethod
    def ipv4_validator(dest):
        """IPv4 address validator."""
        try:
            ip = socket.gethostbyname(dest)
        except socket.error:
            return None
        return ip

    def next_seq(self):
        """Returns next sequence number."""
        if self.last_recv_seq != self.v_seq_num:
            self.losted += 1
            self.status = STAT_DOWN
            if not self.outage:
                self.outage = True
                self.outage_start_time = time.time()
            else:
                if not self.logged:
                    self.LOG.info(LOG_DOWN.format(self.name))
                    self.logged = True

        self.seq_num += 1
        self.v_seq_num += 1
        self.v_seq_num %= 65536
        self.send_time = time.time()
        return self.v_seq_num

    def recv(self, packet_seq):
        """Responds on recieve IMCP echo reply with 
        sequence number 'packet_seq'."""
        act_time = time.time()
        if packet_seq == self.v_seq_num:
            self.status = STAT_UP
            self.last_recv_seq = packet_seq
            self.time = int((act_time - self.send_time) * 1000)
            self.complete_time += self.time
            if self.time < self.min_delay:
                self.min_delay = self.time
            if self.time > self.max_delay:
                self.max_delay = self.time
            
            if self.outage:
                if not self.logged:
                    self.LOG.info(LOG_LOST.format(self.name))                    
                    self.outage = False
                else:
                    self.outage = False
                    self.logged = False
                    self.LOG.info(LOG_UP.format(self.name))
                    out_time = (time.time() - self.outage_start_time) / 60.0
                    self.LOG.info(LOG_SUMMARY.format(self.name, out_time))
                
